#pragma once 

#include "stdafx.h"
#include "cameraCtrl.h"

EdsError EDSCALLBACK handleObjectEvent( EdsObjectEvent event, EdsBaseRef object, EdsVoid * context)
{
	EdsError err = EDS_ERR_OK;
	
	// do something
	
	switch(event)
	{
		case kEdsObjectEvent_DirItemRequestTransfer:
		downloadImage(object);
		break;

		default:
		break;
	}

	// Object must be released
	if(object)
	{
		EdsRelease(object);
	}

	return err;
}


EdsError EDSCALLBACK handleSateEvent (EdsPropertyEvent event, EdsPropertyID property, EdsVoid * context)
{
	//EdsError EDSCALLBACK handleSateEvent (EdsCameraStateEvent event, EdsUInt32 parameter, EdsVoid * context);
	EdsError err = EDS_ERR_OK;
	return err;
}


EdsError EDSCALLBACK handlePropertyEvent (EdsPropertyEvent event, EdsPropertyID property, EdsVoid * context)
{
	//EdsError EDSCALLBACK handleSateEvent (EdsCameraStateEvent event, EdsUInt32 parameter, EdsVoid * context)
	EdsError err = EDS_ERR_OK;
	return err;
}


EdsError getFirstCamera(EdsCameraRef *camera)
{
	EdsError err = EDS_ERR_OK;
	EdsCameraListRef cameraList = NULL;
	EdsUInt32 count = 0;

	// Get camera list
	err = EdsGetCameraList(&cameraList);

	// Get number of cameras
	if(err == EDS_ERR_OK)
	{
		err = EdsGetChildCount(cameraList, &count);
		if(count == 0)
		{
			err = EDS_ERR_DEVICE_NOT_FOUND;
		}
	}
	// Get first camera retrieved
	if(err == EDS_ERR_OK)
	{
		err = EdsGetChildAtIndex(cameraList , 0 , camera);
	}
	// Release camera list
	if(cameraList != NULL)
	{
		EdsRelease(cameraList);
		cameraList = NULL;
	}

	return err;
}

EdsError downloadImage(EdsDirectoryItemRef directoryItem)
{
	EdsError err = EDS_ERR_OK;
	EdsStreamRef stream = NULL;
	// Get directory item information
	EdsDirectoryItemInfo dirItemInfo;
	err = EdsGetDirectoryItemInfo(directoryItem, & dirItemInfo);
	// Create file stream for transfer destination
	if(err == EDS_ERR_OK)
	{
		err = EdsCreateFileStream( dirItemInfo.szFileName, kEdsFileCreateDisposition_CreateAlways, kEdsAccess_ReadWrite, &stream);
	}
	// Download image
	if(err == EDS_ERR_OK)
	{
		err = EdsDownload( directoryItem, dirItemInfo.size, stream);
	}
	// Issue notification that download is complete
	if(err == EDS_ERR_OK)
	{
		err = EdsDownloadComplete(directoryItem);
	}
	// Release stream
	if( stream != NULL)
	{
		EdsRelease(stream);
		stream = NULL;
	}
	return err;
}


EdsError takePicture(EdsCameraRef camera)
{
	return EdsSendCommand(camera, kEdsCameraCommand_TakePicture , 0);
}


EdsError setTv(EdsCameraRef camera, EdsUInt32 TvValue)
{
	return EdsSetPropertyData(camera, kEdsPropID_Tv, 0 , sizeof(TvValue), &TvValue);
}


EdsError setAv(EdsCameraRef camera, EdsUInt32 AvValue)
{
	return EdsSetPropertyData(camera, kEdsPropID_Av, 0 , sizeof(AvValue), &AvValue);
}


EdsError setISO(EdsCameraRef camera, EdsUInt32 IsoValue)
{
	return EdsSetPropertyData(camera, kEdsPropID_ISOSpeed, 0 , sizeof(IsoValue), &IsoValue);
}


EdsError setImageQuality(EdsCameraRef camera, EdsUInt32 imgQualVal)
{
	return EdsSetPropertyData(camera, kEdsPropID_ImageQuality, 0 , sizeof(imgQualVal), &imgQualVal);
}


EdsError setWhiteBallance(EdsCameraRef camera, EdsUInt32 whiteBallanceVal)
{
	return EdsSetPropertyData(camera, kEdsPropID_WhiteBalance, 0 , sizeof(whiteBallanceVal), &whiteBallanceVal);
}


EdsError setMeterMode(EdsCameraRef camera, EdsUInt32 meterModeVal)
{
	return EdsSetPropertyData(camera, kEdsPropID_MeteringMode, 0 , sizeof(meterModeVal), &meterModeVal);
}

EdsError getAEMode(EdsCameraRef camera, EdsPropertyDesc *AEMode)
{
	EdsError err = EDS_ERR_OK;
	EdsDataType dataType;
	EdsUInt32 dataSize;

	err = EdsGetPropertySize(camera, kEdsPropID_AEMode, 0 , &dataType, &dataSize);

	if(err == EDS_ERR_OK)
	{
		err = EdsGetPropertyData(camera, kEdsPropID_AEMode, 0 , dataSize, AEMode);
	}

	return err;
}

EdsError setStateEventHandler(EdsCameraRef camera, EdsPropertyEventHandler inPropertyEventHandler)
{
	EdsError err = NULL;
	err = EdsSetPropertyEventHandler(camera, kEdsPropertyEvent_All, inPropertyEventHandler, NULL);

	return err;
}

