#include "stdafx.h"
#include "oglInterface.h"
#include "glFuncs.h"
#include <math.h>

oglInterface::oglInterface(void)
{
	m_posX = 0.0f;		// X position of model in camera view
	m_posY = 0.0f;		// Y position of model in camera view
	m_zoom = 120.0f;	// Zoom on model in camera view
	m_rotX = 45.0f;		// Rotation on model in camera view
	m_rotY	= 45.0f;		// Rotation on model in camera view
	m_bIsMaximized = false;

	currFrame = 0;
	numPics = 12;
	stepVal = 2;
	focalLength = 18;
	overlap = 0.1; // 10% overlap
}

oglInterface::~oglInterface(void)
{
	outFile.close();
}

BEGIN_MESSAGE_MAP(oglInterface, CWnd)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_TIMER()
	ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()

void oglInterface::OnPaint()
{
	//CPaintDC dc(this); // device context for painting
	ValidateRect(NULL);
}

void oglInterface::OnSize(UINT nType, int cx, int cy)
{
	CWnd::OnSize(nType, cx, cy);

	if (0 >= cx || 0 >= cy || nType == SIZE_MINIMIZED) return;

	// Map the OpenGL coordinates.
	glViewport(0, 0, cx, cy);

	// Projection view
	glMatrixMode(GL_PROJECTION);

	glLoadIdentity();

	// Set our current view perspective
	gluPerspective(30.0f, (float)cx / (float)cy, 0.01f, 2000.0f);

	// Model view
	glMatrixMode(GL_MODELVIEW);

	switch (nType)
	{
		// If window resize token is "maximize"
		case SIZE_MAXIMIZED:
		{
			// Get the current window rect
			GetWindowRect(m_rect);

			// Move the window accordingly
			MoveWindow(6, 6, cx - 14, cy - 14);

			// Get the new window rect
			GetWindowRect(m_rect);

			// Store our old window as the new rect
			m_oldWindow = m_rect;

			break;
		}

		// If window resize token is "restore"
		case SIZE_RESTORED:
		{
			// If the window is currently maximized
			if (m_bIsMaximized)
			{
				// Get the current window rect
				GetWindowRect(m_rect);

				// Move the window accordingly (to our stored old window)
				MoveWindow(m_oldWindow.left, m_oldWindow.top - 18, m_originalRect.Width() - 4, m_originalRect.Height() - 4);

				// Get the new window rect
				GetWindowRect(m_rect);

				// Store our old window as the new rect
				m_oldWindow = m_rect;
			}
		
			break;
		}
	}
}

BOOL oglInterface::PreTranslateMessage(MSG* pMsg) 
{
	switch (pMsg->message)
	{
		case WM_KEYUP:
			//currFrame++;
			break;
		
		case WM_KEYDOWN:
			break;
	}
	return CWnd::PreTranslateMessage(pMsg);
}


int oglInterface::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CWnd::OnCreate(lpCreateStruct) == -1) return -1;

	oglInitialize();

	return 0;
}

void oglInterface::OnDraw(CDC *pDC)
{
	glLoadIdentity();

	glTranslatef(0.0f, 0.0f, -m_zoom);
	glTranslatef(m_posX, m_posY, 0.0f);
	glRotatef(m_rotX, 1.0f, 0.0f, 0.0f);
	glRotatef(m_rotY, 0.0f, 1.0f, 0.0f);
}

void oglInterface::OnTimer(UINT nIDEvent)
{
	switch (nIDEvent)
	{
		case 1:
		{
			// Clear color and depth buffer bits
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			// Draw OpenGL scene
			oglDrawScene();

			// Swap buffers
			SwapBuffers(hdc);

			break;
		}

		default:
			break;
	}

	CWnd::OnTimer(nIDEvent);
}

void oglInterface::OnMouseMove(UINT nFlags, CPoint point)
{
	int diffX = (int)(point.x - m_lastX);
	int diffY = (int)(point.y - m_lastY);
	m_lastX  = (float)point.x;
	m_lastY  = (float)point.y;

	// Left mouse button
	if (nFlags & MK_LBUTTON)
	{
		m_rotX += (float)0.5f * diffY;
		m_rotY += (float)0.5f * diffX;
	}

	OnDraw(NULL);

	CWnd::OnMouseMove(nFlags, point);
}

void oglInterface::oglCreate(CRect rect, CWnd *parent)
{
	CString className = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW | CS_OWNDC, NULL, (HBRUSH)GetStockObject(BLACK_BRUSH), NULL);

	CreateEx(0, className, _T("OpenGL"), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, rect, parent, 0);

	// Set initial variables' values
	m_oldWindow	   = rect;
	m_originalRect = rect;

	hWnd = parent;
}

void oglInterface::oglInitialize(void)
{
	// Initial Setup:
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		32, // bit depth
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		16, // z-buffer depth
		0, 0, 0, 0, 0, 0, 0,
	};

	// Get device context only once.
	hdc = GetDC()->m_hDC;
	
	// Pixel format.
	m_nPixelFormat = ChoosePixelFormat(hdc, &pfd);
	SetPixelFormat(hdc, m_nPixelFormat, &pfd);

	// Create the OpenGL Rendering Context.
	hrc = wglCreateContext(hdc);
	wglMakeCurrent(hdc, hrc);

	// Basic Setup:
	//
	// Set color to use when clearing the background.
	glClearColor(0.623f, 0.623f, 0.623f, 1.0f);
	glClearDepth(1.0f);

	// Turn on backface culling
	glFrontFace(GL_CCW);
	glCullFace(GL_BACK);
	
	// Turn on depth testing
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	// Send draw request
	OnDraw(NULL);
}

void oglInterface::oglDrawScene(void)
{
	float picsDiv = 360 / numPics;
	width = 7;
	height = 1.5 * width;
	int count = 0;

	for(int i = 0; i < 360; i +=picsDiv)
	{

		glPushMatrix();
		glColor3f(8.0, 0.0, 0.0);
			glRotatef(i, 0, 1, 0);
			glTranslatef(30, 0, 0);

			if(count != currFrame)
				glColor3f(8.0, 0.0, 0.0);
			else
				glColor3f(0.0, 0.8, 0.0);


			glBegin(GL_QUADS);
				glVertex3f(0, -height, width);
				glVertex3f(0, -height, -width);
				glVertex3f(0, height, -width);
				glVertex3f(0, height, width);
			glEnd();

		glPopMatrix();
		count++;
	}


	glPushMatrix();
	glRotatef((360 / numPics) * currFrame , 0, 1, 0);
	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(0, 0, 0);
		glVertex3f(30, -height, width);
		glVertex3f(30, -height, -width);

		glVertex3f(0, 0, 0);

		glVertex3f(30, height, width);
		glVertex3f(30, height, -width);
		glVertex3f(0, 0, 0);
	glEnd();

	glBegin(GL_LINES);
		glVertex3f(30, height, width);
		glVertex3f(30, -height,width);
	glEnd();

	glBegin(GL_LINES);
		glVertex3f(30, height,-width);
		glVertex3f(30, -height, -width);
	glEnd();

	glPopMatrix();
}

void oglInterface::nextFrame(void)
{
	if(currFrame < numPics) 
		currFrame++;
	else
		currFrame = 1;
}

void oglInterface::previousFrame(void)
{
	if(currFrame > 1 ) 
		currFrame--;
	else
		currFrame = numPics;
}


